#!/usr/bin/env python
import sys
import os
import jinja2
import logging
import argparse
import copy
from .connectionfactory import ConnectionFactory
from .s3fs import S3Fs, S3FsObjectException, S3FsObject
from .kmsresolver import KMSResolver
from botocore.client import Config
from . import __version__

__author__ = "Giuseppe Chiesa"
__copyright__ = "Copyright 2017, Giuseppe Chiesa"
__credits__ = ["Giuseppe Chiesa"]
__license__ = "BSD"
__maintainer__ = "Giuseppe Chiesa"
__email__ = "mail@giuseppechiesa.it"
__status__ = "PerpetualBeta"


class TemplateRenderer(object):
    """
    Renders a template based on S3Fs location
    """
    def __init__(self, template_file, s3fs):
        """

        :param template_file: template file to process
        :param s3fs: S3Fs object
        :type s3fs: S3Fs
        """
        self._template_file = template_file
        self._s3fs = s3fs
        """ :type : S3Fs """

    def render(self, **kwargs):
        """
        Renders the template
        :param kwargs: additional variables to use in the rendering
        :return: content of the rendered template
        :rtype: basestring
        """
        with open(self._template_file, 'rb') as tpl_file:
            tpl_data = tpl_file.read()
        template = jinja2.Template(tpl_data)
        variables = {obj.name: obj for obj in self._s3fs.objects}
        variables.update(kwargs)
        result = template.render(**variables)
        return result


class S3Vault(object):
    """
    Implements a Vault by using S3 as backend and KMS as way to protect the data
    """
    def __init__(self, bucket, path, connection_factory=None):
        """

        :param bucket: bucket
        :param path: path
        :param connection_factory: connection factory
        :type connection_factory: ConnectionFactory
        """
        self.logger = logging.getLogger(self.__class__.__name__)
        self._bucket = bucket
        self._path = path
        self._connection_manager = connection_factory
        if not self._connection_manager:
            self._connection_manager = ConnectionFactory(config=Config(signature_version='s3v4'))

    def put_file(self, src, dest, encryption_key_arn='', key_alias='', role_name=''):
        """
        Upload a file to the S3Vault
        :param src: source file name
        :param dest: destination file name
        :param encryption_key_arn: KMS Key arn to use
        :param key_alias: KMS Key alias to use
        :param role_name: Role from which resolve the key
        :return: metadata of the uploaded object
        :rtype: dict
        """
        s3fs = S3Fs(self._connection_manager, self._bucket, self._path)
        kms_resolver = KMSResolver(self._connection_manager, keyalias=key_alias, role_name=role_name)
        key_arn = encryption_key_arn
        if not encryption_key_arn:
            key_arn = kms_resolver.retrieve_key_arn()
        with open(src, 'rb') as src_file:
            s3fsobj = s3fs.put_object(dest, src_file.read(), key_arn)
            """ :type : S3FsObject """
        return s3fsobj.metadata

    def get_file(self, name):
        """
        Get a file from S3Vault
        :param name: filename
        :return: file content
        :rtype: basestring
        """
        s3fs = S3Fs(self._connection_manager, self._bucket, self._path)
        s3fsobject = s3fs.get_object(name)
        return str(s3fsobject)

    def render_template(self, template_file, **kwargs):
        """
        Renders a template file using the information available in the S3Vault
        :param template_file: file name to use as template
        :param kwargs: additional variables to use in the rendering
        :return: rendered content
        :rtype: basestring
        """
        s3fs = S3Fs(self._connection_manager, self._bucket, self._path)
        template_renderer = TemplateRenderer(template_file, s3fs)
        return template_renderer.render(**kwargs)

    def create_config_property(self, configfile, encryption_key_arn='', key_alias='', role_name=''):
        """
        Create a configuration file in the S3Vault

        :param configfile: configuration file name
        :param encryption_key_arn: KMS Arn to use
        :param key_alias: KMS Alias to use
        :param role_name: Role to use to resolve the KMS Key
        :return: s3fsobject
        :rtype: S3FsObject
        """
        self.logger.info('Creating new config file: {c}'.format(c=configfile))
        s3fs = S3Fs(self._connection_manager, self._bucket, self._path)
        kms_resolver = KMSResolver(self._connection_manager, keyalias=key_alias, role_name=role_name)
        key_arn = encryption_key_arn
        if not encryption_key_arn:
            key_arn = kms_resolver.retrieve_key_arn()
        s3fsobject = s3fs.put_object(configfile, '{}', key_arn)
        return s3fsobject

    def set_property(self, configfile, key, value, encryption_key_arn='', key_alias='', role_name=''):
        """
        Set a property in a configuration file in the S3Vault

        :param configfile: configfile name
        :param key: key
        :param value: value
        :param encryption_key_arn: KMS Key to use
        :param key_alias: KMS alias to use
        :param role_name: Role to use to resolve the KMS Key
        :return: metadata of the config file created/updated
        :rtype: basestring
        """
        s3fs = S3Fs(self._connection_manager, self._bucket, self._path)
        try:
            s3fsobject = s3fs.get_object(configfile)
        except S3FsObjectException:
            s3fsobject = self.create_config_property(configfile, encryption_key_arn, key_alias, role_name)
        s3fsobject[key] = value
        s3fsobj = s3fs.update_s3fsobject(s3fsobject)
        return s3fsobj.metadata

    def get_property(self, configfile, key):
        """
        Get a configuration property from a config file from the S3Vault
        :param configfile: configuration file
        :param key: key to query
        :return: value of the key
        """
        s3fs = S3Fs(self._connection_manager, self._bucket, self._path)
        try:
            s3fsobject = s3fs.get_object(configfile)
        except:
            self.logger.exception('No configuration with name: {c} found'.format(c=configfile))
            raise
        return s3fsobject[key]


def check_args():
    """
    Check the args from the command line
    :return: args object
    """
    parser = argparse.ArgumentParser(prog='s3vaultcli', description='s3vaultcli', version=__version__)
    parser.add_argument('-b', '--bucket', dest='bucket', required=True,
                        help='Bucket to use for S3Vault')
    parser.add_argument('-p', '--path', dest='path', required=True,
                        help='Path to use in the bucket')
    parser.add_argument('-k', '--kms-alias', dest='kms_alias', required=False,
                        help='Key alias to use to decrypt data')
    parser.add_argument('-L', '--log-level', dest='log_level', required=False,
                        help='Log level to set',
                        choices=['debug', 'info', 'warning', 'error'],
                        default='info')
    parser.add_argument('--profile', dest='profile', required=False,
                        help='AWS profile to use',
                        default=None)
    parser.add_argument('--region', dest='region', required=False,
                        help='AWS region to use',
                        default=None)

    subparsers = parser.add_subparsers(dest='command')
    template = subparsers.add_parser('template', help='Expand a template file based on a S3Vault')
    template.add_argument('-t', '--template', dest='template', required=True,
                          help='Template to expand from s3vault path',
                          type=argparse.FileType('rb'))
    template.add_argument('-d', '--dest', dest='dest', required=True,
                          help='Destination file',
                          type=argparse.FileType('wb'))

    pushfile = subparsers.add_parser('push', help='Push a file in the S3Vault')
    pushfile.add_argument('-s', '--src', dest='src', required=True,
                          help='Source file to upload',
                          type=argparse.FileType('rb'))
    pushfile.add_argument('-d', '--dest', dest='dest', required=True,
                          help='Destination name')

    setproperty = subparsers.add_parser('configset', help='Set a property in a configuration file in the S3Vault')
    setproperty.add_argument('-c', '--config', dest='config', required=True,
                             help='Configuration file to manage')
    setproperty.add_argument('-K', '--key', dest='key', required=True,
                             help='Key to set')
    setproperty.add_argument('-V', '--value', dest='value', required=True,
                             help='Value to set')
    ansible = subparsers.add_parser('ansible_path', help='Resolve the ansible module path')

    return parser.parse_args()


def configure_logging(level):
    """
    Configure the logging level of the tool
    :param level: level to set
    :return:
    """
    formatter = '[%(name)s] [%(levelname)s] : %(message)s'
    # formatter = '[%(asctime)s] [%(levelname)s] [%(name)s] [%(message)s]'
    logging.basicConfig(level=logging.getLevelName(level.upper()),
                        format=formatter)


def main():
    """
    Command line tool to use some functionality of the S3Vault
    :return:
    """
    args = check_args()
    configure_logging(args.log_level)
    logger = logging.getLogger(__name__)
    conn_manager = ConnectionFactory(region=args.region, profile=args.profile)

    s3vault = S3Vault(args.bucket, args.path, connection_factory=conn_manager)

    if args.command == 'template':
        try:
            # expose the environemnt variables in 2 dicts
            ansible_env = copy.deepcopy(os.environ)
            environment = copy.deepcopy(os.environ)
            args.dest.write(s3vault.render_template(args.template.name,
                                                    ansible_env=ansible_env,
                                                    environment=environment))
        except Exception as e:
            logger.exception('Error while expanding the template. Exiting')
            sys.exit(1)
    elif args.command == 'push':
        try:
            logger.info(args.src.name)
            metadata = s3vault.put_file(src=args.src.name,
                                        dest=args.dest,
                                        key_alias=args.kms_alias)
            logger.debug('s3fsobject metadata: {d}'.format(d=metadata))
        except Exception as e:
            logger.exception('Error while pushing file. Error: {t} / {e}'.format(t=str(type(e)),
                                                                                 e=str(e)))
            sys.exit(1)
    elif args.command == 'configset':
        try:
            metadata = s3vault.set_property(configfile=args.config,
                                            key=args.key,
                                            value=args.value,
                                            key_alias=args.kms_alias)
            logger.debug('s3fsobject metadata: {d}'.format(d=metadata))
        except Exception as e:
            logger.exception('Error while setting property. Error: {t} / {e}'.format(t=str(type(e)),
                                                                                     e=str(e)))
    elif args.command == 'ansible_path':
            dirname = os.path.dirname(os.path.abspath(__file__))
            print('{}'.format(os.path.join(dirname, 'ansible')))
    else:
        logger.error('Command not available')
        sys.exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import requests
import logging

__author__ = "Giuseppe Chiesa"
__copyright__ = "Copyright 2017, Giuseppe Chiesa"
__credits__ = ["Giuseppe Chiesa"]
__license__ = "BSD"
__maintainer__ = "Giuseppe Chiesa"
__email__ = "mail@giuseppechiesa.it"
__status__ = "PerpetualBeta"


class EC2MetadataException(Exception):
    pass


class EC2Metadata(object):
    """
    Object that retrieve metadata from within an EC2 instance
    """
    def __init__(self, endpoint='169.254.169.254', version='latest'):
        self.logger = logging.getLogger(self.__class__.__name__)
        self._endpoint = endpoint
        self._version = version
        self._instance_identity_document = None
        self._uri = 'http://{e}/{v}'.format(e=endpoint, v=version)

    def _get_data(self, url_path):
        """
        Query the metadata
        """
        url = '{b}/{p}'.format(b=self._uri, p=url_path)
        try:
            response = requests.get(url, timeout=5)
        except Exception:
            self.logger.exception('Error while getting metadata')
            raise
        if not response.ok:
            raise EC2MetadataException('Error while reading metadata from path')
        return response.text.strip()

    @property
    def role(self):
        """
        Return the role associated to the instance
        """
        data = self._get_data('meta-data/iam/security-credentials/')
        if not data:
            raise EC2MetadataException('Role not associated')
        return data

    @property
    def account_id(self):
        """
        Return the account_id associated to the instance
        :return: account_id
        :rtype: basestring
        """
        return self._get_instance_identity_document()['accountId']

    @property
    def region(self):
        """
        Return the region associated to the instance
        :return: region
        :rtype: basestring
        """
        return self._get_instance_identity_document()['availabilityZone'][-1]

    @property
    def instance_id(self):
        """
        Return the instance_id associated to the instance
        :return: instance_id
        :rtype: basestring
        """
        return self._get_instance_identity_document()['instanceId']

    def _get_instance_identity_document(self):
        if not self._instance_identity_document:
            data = self._get_data('latest/dynamic/instance-identity/document')
            if not data:
                raise EC2MetadataException('Unable to retrieve instance identity document')
            self._instance_identity_document = data
        return self._instance_identity_document



#!/usr/bin/env python
from .ec2metadata import EC2Metadata

__author__ = "Giuseppe Chiesa"
__copyright__ = "Copyright 2017, Giuseppe Chiesa"
__credits__ = ["Giuseppe Chiesa"]
__license__ = "BSD"
__maintainer__ = "Giuseppe Chiesa"
__email__ = "mail@giuseppechiesa.it"
__status__ = "PerpetualBeta"


class KMSResolverException(Exception):
    pass


class KMSResolver(object):
    """
    Object that resolves the KMS key associated to a role, or
    load a keyarn with a specified alias
    """
    def __init__(self, connection_manager, keyalias='', role_name=''):
        self._connmanager = connection_manager
        """ :type : ConnectionManager """
        self._keyalias = keyalias
        self._role = role_name
        self._kms = self._connmanager.client('kms')
        """ :type : pyboto3.kms """

    def _get_key_from_alias(self, alias):
        key_id = 'alias/{a}'.format(a=alias.rpartition('alias/')[-1])
        key_data = self._kms.describe_key(KeyId=key_id)
        if not key_data.get('KeyMetadata'):
            return ''
        return key_data['KeyMetadata'].get('Arn')

    def retrieve_key_arn(self):
        """
        Return the KMS arn of a key
        :return: key arn
        :rtype: basestring
        """
        key_arn = ''
        if self._keyalias:
            key_arn = self._get_key_from_alias(self._keyalias)

        if key_arn:
            return key_arn

        if self._role:
            key_arn = self._get_key_from_alias(self._role)

        if key_arn:
            return key_arn

        role = EC2Metadata().role
        key_arn = self._get_key_from_alias(role)
        if not key_arn:
            raise KMSResolverException('Unable to resolve the key')
        return key_arn



#!/usr/bin/env python
import jinja2
import os
import copy
import logging
from pwd import getpwnam
from grp import getgrnam
from ansible.module_utils.basic import AnsibleModule
from s3vaultlib.ec2metadata import EC2Metadata, EC2MetadataException
from s3vaultlib.connectionfactory import ConnectionFactory
from s3vaultlib.s3vaultlib import S3Vault

logging.basicConfig(level=logging.DEBUG)

__author__ = "Giuseppe Chiesa"
__copyright__ = "Copyright 2017, Giuseppe Chiesa"
__credits__ = ["Giuseppe Chiesa"]
__license__ = "BSD"
__maintainer__ = "Giuseppe Chiesa"
__email__ = "mail@giuseppechiesa.it"
__status__ = "PerpetualBeta"


ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = '''
---
module: s3vault_template

short_description: Expand a template from a S3Vault

version_added: "2.4"

description:
    - "The module allow expand a template from s3vault"

        bucket=dict(type='str', required=True),
        path=dict(type='str', required=False, default='role/{{ role_name }}/encrypted'),
        kms_alias=dict(type='str', required=False),
        region=dict(type='str', required=True, default=None),
        profile=dict(type='str', required=False, default=None),
        src=dict(type='str', required=True),
        dest=dict(type='str', required=True),

options:
    bucket: 
        description:
            - The S3 bucket where the S3Vault resides
    path:
        description:
            - The path to use inside the bucket (note this path support variable expansion, like {{ role }} )
    kms_alias:
        description:
            - The KMS Alias to use to decrypt the data. If not specified the module will try to use the name of
              the role associate to the instance as KMS alias
    region:
        description:
            - The region to use
    profile: 
        description: 
            - Profile to use            
    mode: 
        description: 
            - Mode (numeric) to associate to the created destination file (default 0666)             
    owner: 
        description: 
            - User to associate to the created destination file             
    group: 
        description: 
            - Group to associate to the created destination file               
    src:        
        description: 
            - The source template to expand
    dest:        
        description: 
            - The destination file where to expand the template

author:
    - Giuseppe Chiesa (@gchiesa)
'''

EXAMPLES = '''
# Expand a template based on a configuration for nginx
- name: Expand nginx configuration
  s3vault_template:
    bucket: 123456789-vault
    region: eu-west-1
    src: /etc/nginx/nginx.conf.j2
    dest: /etc/nginx/nginx.conf
'''

RETURN = '''
original_message:
    description: The original name param that was passed in
    type: str
message:
    description: Template: {src} expanded to: {dest}
'''


def run_module():
    # define the available arguments/parameters that a user can pass to
    # the module
    module_args = dict(
        bucket=dict(type='str', required=True),
        path=dict(type='str', required=False, default='role/{{ role_name }}/encrypted'),
        kms_alias=dict(type='str', required=False),
        region=dict(type='str', required=False, default=None),
        profile=dict(type='str', required=False, default=None),
        src=dict(type='str', required=True),
        dest=dict(type='str', required=True),
        mode=dict(type='str', required=False, default='0666'),
        owner=dict(type='str', required=False, default='root'),
        group=dict(type='str', required=False, default='root'),
        ec2=dict(type='bool', required=False, default=True)
    )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
        original_message='',
        message=''
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True
    )

    # if the user is working with this module in only check mode we do not
    # want to make any changes to the environment, just return the current
    # state with no modifications
    if module.check_mode:
        return result

    # get the role
    if module.params['ec2']:
        ec2_metadata = EC2Metadata()
        template = jinja2.Template(module.params['path'])
        vault_path = template.render({'role_name': ec2_metadata.role})
    else:
        vault_path = module.params['path']

    conn_manager = ConnectionFactory(region=module.params['region'], profile_name=module.params['profile'])
    s3vault = S3Vault(module.params['bucket'], vault_path, connection_factory=conn_manager)

    ansible_env = copy.deepcopy(os.environ)
    environment = copy.deepcopy(os.environ)

    dest_path = os.path.dirname(os.path.abspath(module.params['dest']))
    if not os.access(dest_path, os.W_OK):
        module.fail_json(msg='Unable to write the destination file', **result)

    if not os.access(module.params['src'], os.R_OK):
        module.fail_json(msg='Unable to read from the source file', **result)

    with open(module.params['dest'], 'wb') as f_handler:
        f_handler.write(s3vault.render_template(module.params['src'],
                                                ansible_env=ansible_env,
                                                environment=environment))

    os.chmod(module.params['dest'], module.params['mode'])

    # resolve user and group
    group_id = getgrnam(module.params['group']).gr_gid
    user_id = getpwnam(module.params['owner'].pw_uid)
    os.chown(module.params['dest'], user_id, group_id)

    result['changed'] = True
    result['message'] = 'Template: {src} expanded to: {dest}'.format(src=module.params['src'],
                                                                     dest=module.params['dest'])
    module.exit_json(**result)


def main():
    run_module()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import logging
import json
import os
from .connectionfactory import ConnectionFactory

__author__ = "Giuseppe Chiesa"
__copyright__ = "Copyright 2017, Giuseppe Chiesa"
__credits__ = ["Giuseppe Chiesa"]
__license__ = "BSD"
__maintainer__ = "Giuseppe Chiesa"
__email__ = "mail@giuseppechiesa.it"
__status__ = "PerpetualBeta"


MAX_S3_RETURNED_OBJECTS = 999


class S3FsException(Exception):
    pass


class S3Fs(object):
    """
    Object that abstracts operation with encrypted objects on S3
    """
    def __init__(self, connection_factory, bucket, path=''):
        """

        :param connection_factory: connection_factory object
        :type connection_factory: ConnectionFactory
        :param bucket: S3 bucket
        :param path: bucket path
        """
        self._connection_factory = connection_factory
        """ :type : ConnectionFactory """
        self.logger = logging.getLogger(self.__class__.__name__)
        self._bucket = bucket
        self._path = path
        self._s3fs_objects = []
        self.fs = self._connection_factory.client('s3')
        """ :type : pyboto3.s3 """

    @staticmethod
    def is_file(s3elem):
        """
        Return true is an s3 json element represents a valid file
        """
        try:
            key = s3elem['Key']
        except KeyError:
            return False
        name = key.rpartition('/')[-1]
        if not name:
            return False
        return True

    @property
    def objects(self):
        """
        Return a list of s3fsobjects
        """
        return self._get_s3fsobjects()

    def _get_s3fsobjects(self, refresh=False):
        """
        load the s3fsobjects from an S3 path
        :param refresh: True to reload the objects
        :return: list of object
        :rtype: list
        """
        if self._s3fs_objects and not refresh:
            return self._s3fs_objects
        response = self.fs.list_objects_v2(Bucket=self._bucket,
                                           Prefix=self._path,
                                           MaxKeys=MAX_S3_RETURNED_OBJECTS)
        if not response.get('Contents'):
            self._s3fs_objects = []
            return self._s3fs_objects

        for elem in response.get('Contents'):
            if self.is_file(elem):
                self._s3fs_objects.append(S3FsObject(elem, self._bucket, self._path, self.fs))
        return self._s3fs_objects

    def get_object(self, name):
        """
        Return a s3fsobject identified by name
        :param name: object name
        :return: s3fsobject
        :rtype: S3FsObject
        """
        s3obj = next(iter([s3fsobj for s3fsobj in self.objects if s3fsobj.name == name]), None)
        if not s3obj:
            raise S3FsObjectException('Object not found')
        return s3obj

    def put_object(self, name, content, encryption_key_arn):
        """
        Put an object in the S3 path by encrypting it with SSE
        :param name: object name
        :param content: content of the object
        :param encryption_key_arn: key arn to use for encryption
        :return: the created s3object
        :rtype: S3FsObject
        """
        if '.' in name:
            raise ValueError('object does not support . (dot) in the name')

        self.logger.info('Adding object: {n}, size: {s}, to bucket: {b}, path: {p}'.format(n=name,
                                                                                           s=len(content),
                                                                                           b=self._bucket,
                                                                                           p=self._path))
        self.fs.put_object(Bucket=self._bucket,
                           ServerSideEncryption='aws:kms',
                           Body=content,
                           Key=os.path.join(self._path, name),
                           SSEKMSKeyId=encryption_key_arn)
        s3obj = next(iter([s3fsobj for s3fsobj in self._get_s3fsobjects(refresh=True) if s3fsobj.name == name]), None)
        return s3obj

    def update_s3fsobject(self, s3fsobject):
        """
        Update an S3FSObject
        :param s3fsobject: S3FsObject to update
        :type: S3FsObject
        :return: the updated object
        :rtype: S3FsObject
        """
        if not s3fsobject.is_encrypted:
            raise S3FsException('Unable to update unencrypted object')
        return self.put_object(s3fsobject.name, str(s3fsobject), s3fsobject.kms_arn)


class S3FsObjectException(Exception):
    pass


class S3FsObject(object):
    """
    Implement the S3FsObject, an abstraction around a S3 file with SSE encryption
    """
    def __init__(self, data, bucket, path, fs):
        """

        :param data: json metadata from the file
        :param bucket: bucket
        :param path: path
        :param fs: s3 cient
        """
        self.logger = logging.getLogger(self.__class__.__name__)
        self._data = data
        self._header = {}
        self._bucket = bucket
        self._path = path
        self._raw = ''
        self._is_json = False
        self._fs = fs
        """ :type : pyboto3.s3 """
        if not self._data.get('Key'):
            raise S3FsObjectException('Not a valid object')
        self.name = self._data['Key'].rpartition('/')[-1]

    @property
    def kms_arn(self):
        """
        Return the KMS ARN used to encrypt the object
        :return: KMS ARN
        :rtype: basestring
        """
        return self._header.get('SSEKMSKeyId', '')

    @property
    def is_encrypted(self):
        """
        Return true if the object is encrypted
        :return: True or False
        :rtype: bool
        """
        if self.kms_arn:
            return True
        return False

    @property
    def metadata(self):
        """
        Return the metadata associated with the object (file metadata)
        :return: medatata
        :rtype: dict
        """
        return self._data

    def _load_content(self):
        """
        Load the content of the file pointed by S3FsObject
        :return: content of the file
        """
        object_path = os.path.join(self._path, self.name)
        try:
            self._header = self._fs.head_object(Bucket=self._bucket, Key=object_path)
        except Exception:
            self.logger.exception('Exception while fetching header for key: {k}'.format(k=object_path))
            raise

        response = self._fs.get_object(Bucket=self._bucket, Key=object_path)
        if not response.get('Body'):
            raise S3FsObjectException('Unable to read the file content for key: {k}'.format(k=object_path))
        self._raw = response['Body'].read()
        return self._raw

    @staticmethod
    def is_json(data):
        """
        Return True if the content is a valid json
        :param data: content to evaluate
        :return: True or False
        :rtype: bool
        """
        try:
            json.loads(data)
        except ValueError:
            return False
        return True

    def __getitem__(self, item):
        """
        Overrides the getitem method
        :param item:
        :return:
        """
        if not self._raw:
            self._load_content()

        if not self.is_json(self._raw):
            raise KeyError(item)

        json_data = json.loads(self._raw)
        if json_data.get(item, ''):
            return json_data[item]
        raise KeyError(item)

    def __setitem__(self, key, value):
        """
        Overrides the setitem method
        :param key:
        :param value:
        :return:
        """
        if not self._raw:
            self._load_content()

        if not self.is_json(self._raw):
            raise KeyError(key)

        json_data = json.loads(self._raw)
        json_data[key] = value
        self._raw = json.dumps(json_data)

    def __getattr__(self, item):
        """
        Override the setattr method
        :param item:
        :return:
        """
        try:
            return self.__getitem__(item)
        except KeyError:
            raise AttributeError(item)

    def __str__(self):
        """
        Override the str method
        :return:
        """
        if not self._raw:
            self._load_content()
        return self._raw



#!/usr/bin/env python
import boto3
import logging
from .ec2metadata import EC2Metadata

__author__ = "Giuseppe Chiesa"
__copyright__ = "Copyright 2017, Giuseppe Chiesa"
__credits__ = ["Giuseppe Chiesa"]
__license__ = "BSD"
__maintainer__ = "Giuseppe Chiesa"
__email__ = "mail@giuseppechiesa.it"
__status__ = "PerpetualBeta"


class ConnectionFactory(object):
    """
    Object that allocate connection by supporting also connection profile and extended paramaters
    """
    def __init__(self, region=None, endpoint=None, **params):
        self.logger = logging.getLogger(self.__class__.__name__)
        self._region = region
        self._endpoint = endpoint
        self._params = params

    def client(self, resource):
        """Returns a client connection"""
        return self._connection('client', resource)

    def _connection(self, type=None, resource=None):
        """Allocate a connection"""
        profile = self._params.pop('profile_name', None)
        if not self._region:
            self._region = EC2Metadata().region

        if type not in ['both', 'resource', 'client']:
            raise ValueError('connection: {c} not supported'.format(c=type))

        if type == 'resource':
            resource = boto3.session.Session(profile_name=profile).resource(resource,
                                                                            region_name=self._region,
                                                                            endpoint_url=self._endpoint,
                                                                            **self._params)
            return resource
        elif type == 'client':
            client = boto3.session.Session(profile_name=profile).client(resource,
                                                                        region_name=self._region,
                                                                        endpoint_url=self._endpoint,
                                                                        **self._params)
            return client
        else:
            client = boto3.session.Session(profile_name=profile).client(resource,
                                                                        region_name=self._region,
                                                                        endpoint_url=self._endpoint,
                                                                        **self._params)
            resource = boto3.session.Session(profile_name=profile).resource(resource,
                                                                            region_name=self._region,
                                                                            endpoint_url=self._endpoint,
                                                                            **self._params)
            return client, resource



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as requirements_file:
    requirements = [line.strip() for line in requirements_file.readlines()]

setup_requirements = [
    'pytest-runner',
    # TODO(s3vaultlib): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    'pytest',
    # TODO: put package test requirements here
]

setup(
    name='s3vaultlib',
    version='1.1.0',
    description="Python library to expose S3 as vault to store encrypted data",
    long_description=readme + '\n\n' + history,
    author="Giuseppe Chiesa",
    author_email='mail@giuseppechiesa.it',
    url='https://github.com/gchiesa/s3vaultlib',
    packages=find_packages(include=['s3vaultlib']),
    include_package_data=True,
    install_requires=requirements,
    license="BSD license",
    zip_safe=False,
    keywords='s3vaultlib',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
    entry_points={
        'console_scripts': [
            's3vaultcli = s3vaultlib.s3vaultlib:main'
        ]
    }
)
